@@ -107,7 +107,7 @@ add_task(async function getOrCreate() {
   const nonexistentDir = PathUtils.join(PathUtils.profileDir, "nonexistent");
   await Assert.rejects(
     KeyValueService.getOrCreate(nonexistentDir, "db"),
-    /UnsuitableEnvironmentPath/
+    /NotFound/
   );
 
   // Test creating a database with a non-normalized but fully-qualified path.
