@@ -701,10 +701,15 @@ add_task(async function test_normalized_optional_origins() {
 
   await assertAddonWrapperPermissionsProperties(manifestV2, {
     isPrivileged: false,
-    userPermissions: { permissions: manifestV2.permissions, origins: [] },
+    userPermissions: {
+      permissions: manifestV2.permissions,
+      origins: [],
+      data_collection: [],
+    },
     optionalPermissions: {
       permissions: [],
       origins: manifestV2.optional_permissions,
+      data_collection: [],
     },
     optionalOriginsNormalized: [
       "http://*.example.com/*",
@@ -745,10 +750,15 @@ add_task(async function test_normalized_optional_origins() {
 
   await assertAddonWrapperPermissionsProperties(manifestV3, {
     isPrivileged: false,
-    userPermissions: { permissions: manifestV3.permissions, origins: [] },
+    userPermissions: {
+      permissions: manifestV3.permissions,
+      origins: [],
+      data_collection: [],
+    },
     optionalPermissions: {
       permissions: [],
       origins: [...manifestV3.host_permissions, "*://*/*"],
+      data_collection: [],
     },
     optionalOriginsNormalized: [
       "http://*.example.com/*",
@@ -778,10 +788,12 @@ add_task(async function test_normalized_optional_origins() {
     userPermissions: {
       permissions: manifestV2Privileged.permissions,
       origins: [],
+      data_collection: [],
     },
     optionalPermissions: {
       permissions: [],
       origins: manifestV2Privileged.optional_permissions,
+      data_collection: [],
     },
     optionalOriginsNormalized: [
       "http://*.example.com/*",
